@@ -17,8 +17,8 @@
 #'
 #' Term Frequency Inverse Document Frequency
 #' @description Creates TfIdf(Latent semantic analysis) model.
-#' The IDF is defined as follows: \code{idf = log(# documents in the corpus) /
-#' (# documents where the term appears + 1)}
+#' "smooth" IDF (default) is defined as follows: \code{idf = log(1 + (# documents in the corpus) / (# documents where the term appears) )}
+#' "non-smooth" IDF is defined as follows: \code{idf = log((# documents in the corpus) / (# documents where the term appears) )}
 #' @format \code{\link{R6Class}} object.
 #' @section Usage:
 #' For usage details see \bold{Methods, Arguments and Examples} sections.
@@ -40,7 +40,7 @@
 #'  \item{x}{An input term-co-occurence matrix. Preferably in \code{dgCMatrix} format}
 #'  \item{smooth_idf}{\code{TRUE} smooth IDF weights by adding one to document
 #'   frequencies, as if an extra document was seen containing every term in the
-#'   collection exactly once. This prevents division by zero.}
+#'   collection exactly once.}
 #'  \item{norm}{\code{c("l1", "l2", "none")} Type of normalization to apply to term vectors.
 #'   \code{"l1"} by default, i.e., scale by the number of words in the document. }
 #'  \item{sublinear_tf}{\code{FALSE} Apply sublinear term-frequency scaling, i.e.,
@@ -56,7 +56,7 @@
 #' dtm_tfidf = model_tfidf$fit_transform(dtm)
 TfIdf = R6::R6Class(
   classname = c("TfIdf"),
-  inherit = mlapiTransformation,
+  inherit = mlapi::mlapiTransformation,
   public = list(
     #----------------------------------------------------------------------------
     # methods
@@ -99,10 +99,13 @@ TfIdf = R6::R6Class(
     get_idf = function(x) {
       # abs is needed for case when dtm is matrix from HashCorpus and signed_hash is used!
       cs = colSums( abs(sign(x) ) )
+      idf_ratio = nrow(x) / (cs)
+      # alternative could be idf = log((nrow(x) - cs + 0.5)/(cs + 0.5))
+      # see "Modern Information Retrieval: A Brief Overview" - http://singhal.info/ieee2001.pdf
       if (private$smooth_idf)
-        idf = log(nrow(x) / (cs + 1 ))
+        idf = log1p(idf_ratio)
       else
-        idf = log(nrow(x) / (cs))
+        idf = log(idf_ratio)
       Diagonal(x = idf)
     }
   )
